import jDisco.*;
import jDisco.Process;

/*
 * Simulation of the aspirin concenteration level in the blood of a person
 * taking variable doses at various preset times.
 * 
 * The model has been taken from
 *
 *     Advanced Continuous Simulation Language (ACSL)
 *     Reference Manual, 1999, (p. A-97 - A-101)
 *     http://www.ACSL.sim.com
 *
 * This is a combined continuous and discrete model. The concentration level varies 
 * continously between events, but changes suddenly as doses are taken.
 */

public class AspirinDosage extends Process {
    static final double r = 0.14;
    
    Variable blood, urine;
    double total;
    
    public void derivatives() {
        blood.rate = -r * blood.state;
        urine.rate =  r * blood.state; 
    }
    
    class TakeDose extends Process {
        TakeDose(double dose) {
            this.dose = dose;
        }
        
        public void actions() {
            total += dose;
            blood.state += dose;
        }
        
        double dose;
    }
            
    public void actions() {
        blood = new Variable(0).start();
        urine = new Variable(0).start();
        start();
        activate(new TakeDose(25), at, 0.0);
        activate(new TakeDose(15), at, 2.0);
        activate(new TakeDose(15), at, 6.0);
        activate(new TakeDose(15), at, 10.0);
        activate(new TakeDose(15), at, 18.0);
        activate(new TakeDose(15), at, 22.0);
        activate(new TakeDose(15), at, 26.0);
        activate(new TakeDose(15), at, 30.0);
        activate(new TakeDose( 0), at, 99.0);
        final Graph graph_blood  = new Graph(" blood ");
        final Graph graph_urine  = new Graph(" urine ");
        final Graph graph_total  = new Graph(" total ");
         class StateReporter extends Reporter {    
            public void actions() {
                graph_blood.add(blood);
                graph_urine.add(urine);
                graph_total.add(total);
            }
        }
        new StateReporter().setFrequency(0.5).start();
        hold(50.0);
        graph_blood.show(); 
        graph_urine.show();
        graph_total.show(); 
    }
    
    public static void main(String[] args) {
        activate(new AspirinDosage());
    }
}
