import jDisco.*;
import jDisco.Process;

/*
 * Simulation of the queuing in a bank.
 *
 * The model is discrete and implements an N/M/1 queueing system.
 * It has one queue (tellerQueue), and both its service time and its 
 * interarrival time follow an exponential distribution.
 */

public class Bank extends Process {
    final static double meanInterArrival = 10;
    final static double meanService = 8;
    final static long maxCustomers = 700;
    
    static Head tellerQueue = new Head();
    static boolean tellerFree = true;
    static Customer customerInService = null;
    
    static Random rand = new Random(7913);
    
    static int numCustomers = 0;
      
    static Tally serviceStat = new Tally("Service Time");
    static Tally waitStat = new Tally("Wait Time");
    static Tally residenceStat = new Tally("Residence Time");
    static Tally interArrivalStat = new Tally("InterArrival Time");
    static Accumulate queueStat = new Accumulate("Queue Length");
  
    public void actions() {
        activate(new Customer());
        hold(10000000);
        report();
    }
    
    void report() {
        Tally.printHeading();
        serviceStat.report();
        waitStat.report();
        residenceStat.report();
        queueStat.report();
        interArrivalStat.report();
    }
    
    class Customer extends Process {
        public void actions() {
            numCustomers++;
            if (numCustomers > maxCustomers) 
                return;
            arrivalTime = time();
            double interArrivalTime = rand.negexp(1/meanInterArrival);
            activate(new Customer(), delay, interArrivalTime);
            interArrivalStat.update(interArrivalTime);
            queueStat.update(tellerQueue.cardinal());
            if (!Bank.tellerFree) {
                wait(tellerQueue);
                out();
                queueStat.update(tellerQueue.cardinal());
            }
            waitStat.update(time() - arrivalTime);
            tellerFree = false;
            double serviceTime = rand.negexp(1/meanService);
            hold(serviceTime);
            serviceStat.update(serviceTime);
            residenceStat.update(time() - arrivalTime);
            if (!tellerQueue.empty()) 
                activate((Customer) tellerQueue.first());
            else
                tellerFree = true;
        }
        
        double arrivalTime;
    }
    
    public static void main(String args[]) {
        long startTime = System.currentTimeMillis();
        activate(new Bank());
        long stopTime = System.currentTimeMillis();
        System.out.println("\nExecution time: " +
                           (stopTime - startTime)/1000.0 + " secs.\n");
    } 
}
