import jDisco.*;
import jDisco.Process;

/*
 * Numerical solution of the nonlinear parablic diffusion equation 
 * on the interval [0,1] given byo
 *
 *    du/dt = d/dx(u du/dx) - u^2
 *
 * Initial and boundary conditions:
 *
 *    u(0,x) = 100, u(t, 0) = 50, u(t,1) = 1 - sin(u).
 *
 * Reference:
 *    Sincovec, R.F., Madsen, N. K.
 *    Software for Nonlinear Partial Differential Equations
 *    ACM Trans. Math. Softw., Vol. 1, No. 3, 1975 (pp. 232 - 260).    
*/ 

public class Diffusion extends Process {
    PDEVariable u;
        
    class U extends PDEVariable {
        U(double xLeft, double xRight, int nPoints) {
            super(xLeft, xRight, nPoints);
        }
        
        public double D() {
            return state() - Math.pow(state(), 2);
        }
        
        public double rate() { return ddx(); }
        
        public double initialState() { return 100; }
        
        public double b1Left()  { return 1; }
        public double b3Left()  { return 50; }
        public double b2Right() { return 1; }
        public double b3Right() { return 1 - Math.sin(state()); }
    } 
    
     public void actions() {
         dtMin = 1.0e-10; dtMax = 1.0e-5; 
         maxAbsError = maxRelError = 1.0e-4;
         u = new U(0, 1, 31);
         u.start();
         hold(0.05);
         u.show();
         hold(0.05);
         u.show();
    }
        
    public static void main(String[] args) {
        activate(new Diffusion());
    }
}


