import jDisco.*;
import jDisco.Process;
import jDisco.Function;

/*
 * The solution of higher-order differential equations normally requires that 
 * the user rewrites the system as first-order equations. The class Function 
 * frees the user from this task.
 *
 * This example uses class Function for numerically solving the second-order 
 * equation of Van der Pol:
 *
 *    d^2y/dt^2 + E(1-y^2)dy/dt + y = 0    
 */

public class VanDerPol extends Process {
    Function y;
    static final double E = -5.0;
    
     public void actions() {
         dtMin = 1.0e-7; dtMax = 0.5; maxRelError = 1.0e-6;
         y = new Function(2);
         y.d(0).state = 0;
         y.d(1).state = 1;
         y.start();
         start();
         final Graph g = new Graph(" Van der Pol ");
         new Reporter() {
            public void actions() {
                g.add(y.d(0));
            }
        }.setFrequency(0.1).start();
        hold(100);
        g.show();
    }
    
    public void derivatives() {
        y.d(2).state = -E * (1 - Math.pow(y.d(0).state, 2)) * y.d(1).state -
                       y.d(0).state; 
    }
        
    public static void main(String[] args) {
        activate(new VanDerPol());
    }
}


